Array.prototype.toJSON = function() {
    return '[' + this.join(', ') + ']';
}

var kEpgFeed            = 0;
var kPopularFeed        = 1;
var kMGUFeed            = 2;
var kMGSFeed            = 3;
var kRecommendedFeed    = 4;
var kUpdateCheckerFeed  = 5;
var kFavoritesFeed      = 6;
var kDevicesFeed        = 7;
var kKeywordsFeed       = 8;
var kNetworksFeed       = 8;

var mainAnimationController = new AnimationController();
var tooltipAnimationController = new AnimationController();
var mainTimer = null;

var lasttimestamp = null;

var bigSpinnerTimer = null;
var bigSpinnerPosition = 0;

var popularScrollArea;
var popularScroller;
var myguideScrollArea;
var myguideScroller;
var myguideSeriesScrollArea;
var myguideSeriesScroller;
var epgScrollArea;
var epgScroller;

var baseSheetHeight;

var initialSetupComplete = false;
var epgLoaded = false;
var popLoaded = false;
var mguLoaded = false;
var mgsLoaded = false;
var rcmLoaded = false;
var favLoaded = false;
var dvLoaded = false;
var chmLoaded = false;
var keyLoaded = false;
var netLoaded = false;

var selectedPage = -1;

var rmtQueue = new RemoteQueue();
var bck = new BackController();
var sht = new SheetManager();
var epg = new Epg();
var dv = new DeviceManager();

var popular = new Popular();
var mgu = new MyGuideUpcoming();
var mgs = new MyGuideSeries();
var chm = new ChannelManager();
var fav = new Favorites();
var rcm = new Recommended();
var upd = new UpdateChecker();
var key = new Keywords();
var net = new Networks();

var epgDownloader       = new FeedDownloader("epg",         function(json){epg.downloaderCallback(json); });
var chmDownloader       = new FeedDownloader("channels",    function(json){chm.downloaderCallback(json); });
var popularDownloader   = new FeedDownloader("popular",     function(json){popular.downloaderCallback(json); });
var mguDownloader       = new FeedDownloader("mgu",         function(json){mgu.downloaderCallback(json); });
var mgsDownloader       = new FeedDownloader("mgs",         function(json){mgs.downloaderCallback(json); });
var favDownloader       = new FeedDownloader("favorites",   function(json){fav.downloaderCallback(json); });
var dvDownloader        = new FeedDownloader("devices",     function(json){dv.downloaderCallback(json); });
var rcmDownloader       = new FeedDownloader("recommended", function(json){rcm.downloaderCallback(json); });
var keyDownloader       = new FeedDownloader("keywords",    function(json){key.downloaderCallback(json); });
var netDownloader       = new FeedDownloader("networks",    function(json){net.downloaderCallback(json); });

var ath = new AuthManager();
var spinnerImage;

function DEBUGLOG(str){
    if(!loggingEnabled)
        return;
        
    var CurrentDate = new Date();
    if (window.widget)
        alert(CurrentDate.format('HH:mm:ss') + '.' + CurrentDate.getMilliseconds() + ': ' + str);
    else
        console.log(CurrentDate.format('HH:mm:ss') + '.' + CurrentDate.getMilliseconds() + ': ' + str);
	return;
}

function animatedBigSpinner(){
	document.getElementById("fontSpinner").style.backgroundPosition = "0px -" + (bigSpinnerPosition * 30) + "px";
  	if(bigSpinnerPosition == 17){
		bigSpinnerPosition = 0;
		return;
	}

	bigSpinnerPosition++;
}

function channelListUpdated(){
//    DEBUGLOG("core.js[103]: channelListUpdated");
	epg.buildDisplay();
	epg.updateDisplay();
	bck.buildChannelLineup();
}

function seriesRecordDataUpdated(){
//    DEBUGLOG("core.js[110]: seriesRecordDataUpdated");
    popular.updateDisplay();
	epg.updateDisplay();
	rcm.updateDisplay();

	if(epg.inSearch && epg.searchString != null){
		epg.doSearch();	
	}

	if(mgu.inSearch && mgu.searchString != null){
		mgu.doSearch();	
	}

	if(mgs.inSearch && mgs.searchString != null){
		mgs.doSearch();	
	}

	if(rcm.inSearch && rcm.searchString != null){
		rcm.doSearch();	
	}
}

function upcomingRecordDataUpdated(){
//    DEBUGLOG("core.js[119]: upcomingRecordDataUpdated");
	popular.updateDisplay();
	epg.updateDisplay();
	mgu.updateDisplay();

	if(epg.inSearch && epg.searchString != null){
		epg.doSearch();	
	}

	if(mgu.inSearch && mgu.searchString != null){
		mgu.doSearch();	
	}

	if(mgs.inSearch && mgs.searchString != null){
		mgs.doSearch();	
	}

	if(rcm.inSearch && rcm.searchString != null){
		rcm.doSearch();	
	}
}

function favoriteRecordDataUpdated(){
//    DEBUGLOG("core.js[131]: favoriteRecordDataUpdated");
	popular.updateDisplay();
	mgs.mergeData();
	mgs.updateDisplay();
	mgs.resize();
	epg.updateDisplay();
	rcm.updateDisplay();
	
	if(epg.inSearch && epg.searchString != null){
		epg.doSearch();	
	}

	if(mgu.inSearch && mgu.searchString != null){
		mgu.doSearch();	
	}

	if(mgs.inSearch && mgs.searchString != null){
		mgs.doSearch();	
	}

	if(rcm.inSearch && rcm.searchString != null){
		rcm.doSearch();	
	}
}

function keywordRecordDataUpdated(){
//    DEBUGLOG("core.js[146]: keywordRecordDataUpdated");
	popular.updateDisplay();
	mgs.mergeData();
	mgs.updateDisplay();
	mgs.resize();
	epg.updateDisplay();
	rcm.updateDisplay();

	if(epg.inSearch && epg.searchString != null){
		epg.doSearch();	
	}

	if(mgu.inSearch && mgu.searchString != null){
		mgu.doSearch();	
	}

	if(mgs.inSearch && mgs.searchString != null){
		mgs.doSearch();	
	}

	if(rcm.inSearch && rcm.searchString != null){
		rcm.doSearch();	
	}
}

function networkDataUpdated(){
//    DEBUGLOG("core.js[161]: networkDataUpdated");
}


function addedSeriesRecording(json){
//    DEBUGLOG("core.js[166]: addedSeriesRecording");
    mgs.updateData(true);
    mgu.updateData(true);
    rcm.updateData(true);
}

function updatedSeriesRecording(json){
//    DEBUGLOG("core.js[176]: updatedSeriesRecording");
    mgs.updateData(true);
    mgu.updateData(true);
    rcm.updateData(true);
}

function updatedKeywordRecording(json){
//    DEBUGLOG("core.js[186]: updatedKeywordRecording");
    key.updateData(true);
    mgs.updateData(true);
    mgu.updateData(true);
	rcm.updateData(true); 
}

function addedShowRecording(json){
//    DEBUGLOG("core.js[197]: addedShowRecording");
    mgs.updateData(true);
    mgu.updateData(true);
}

function addedFav(json){
//    DEBUGLOG("core.js[207]: addedFav");
    mgs.updateData(true);
    mgu.updateData(true);
    fav.updateData(true);
    rcm.updateData(true);
}

function deletedFav(json){
//    DEBUGLOG("core.js[220]: deletedFav");
    mgs.updateData(true);
    mgu.updateData(true);
    fav.updateData(true);
    rcm.updateData(true);
}

function addedKeyword(json){
//    DEBUGLOG("core.js[233]: addedKeyword");
    mgs.updateData(true);
    mgu.updateData(true);
    key.updateData(true);
}

function deletedKeyword(json){
//    DEBUGLOG("core.js[246]: deletedKeyword");
    mgs.updateData(true);
    mgu.updateData(true);
    key.updateData(true);
    rcm.updateData(true);
}


function deletedSeriesRecording(json){
//    DEBUGLOG("core.js[260]: deletedSeriesRecording");
    mgs.updateData(true);
    mgu.updateData(true);
    rcm.updateData(true);
}

function deletedEpisodeRecording(json){
//    DEBUGLOG("core.js[270]: deletedEpisodeRecording");
    mgs.updateData(true);
    mgu.updateData(true);
	rcm.updateData(); 
}

function deletedShowRecording(json){
//    DEBUGLOG("core.js[278]: deletedShowRecording");
    mgu.updateData(true);
	rcm.updateData(true); 
}

function deletedKeywordRecording(json){
//    DEBUGLOG("core.js[285]: deletedKeywordRecording");
    mgu.updateData(true);
	rcm.updateData(true); 
}

function isFavoriteYWE(series_id)
{
	var scheduledRecordingArrayLength = mgu.contentArray.length;
	for(var i=0;i<scheduledRecordingArrayLength;i++){
		if(mgu.contentArray[i]['series_id'] == series_id)
			return mgu.contentArray[i]['is_favorite'];
	}
	return "0";
}

function isPartOfSeriesYWE(series_id)
{
	var scheduledRecordingArrayLength = mgs.contentArray.length;
	for(var i=0;i<scheduledRecordingArrayLength;i++)
		if(mgs.contentArray[i]['series_id'] == series_id)
			return true;
	
	return "0";
}

function isScheduledYWE(show_id)
{
	var scheduledRecordingArrayLength = mgu.contentArray.length;
	for(var i=0;i<scheduledRecordingArrayLength;i++){
		if(mgu.contentArray[i]['show_id'] == show_id)
			return mgu.contentArray[i]['is_scheduled'];
	}
	return "0";
}

function isRecommendedYWE(series_id){
	var recommendedArrayLength = rcm.contentArray.length;
	for(var i=0;i<recommendedArrayLength;i++){
		if(rcm.contentArray[i]['series_id'] == series_id)
			return "1";
	}
	return "0";
}

function isKeywordYWE(show_id){
	var scheduledRecordingArrayLength = mgu.contentArray.length;
    var showFromArray;
    
	for(var i=0;i<scheduledRecordingArrayLength;i++){
        showFromArray = mgu.contentArray[i];
        
		if(mgu.contentArray[i]['show_id'] == show_id)
			return showFromArray['is_keyword'];
	}
	return "0";
}

function keywordViewOrRecordYWE(show_id){
	var scheduledRecordingArrayLength = mgu.contentArray.length;
    var showFromArray;
    
	for(var i=0;i<scheduledRecordingArrayLength;i++){
        showFromArray = mgu.contentArray[i];
        
		if(mgu.contentArray[i]['show_id'] == show_id)
			return showFromArray['view_or_record'];
	}
	return "0";
}

function channelOrderUpdater(){
	epg.updateSortOrder();
}

function updateDayMenu(){
	var days = getFilterDays()
	for(x=0;x<days.length;x++){
		if(x == parseInt(loadPref("currentFilterDay"), 10) && epg.mode == "7"){
			document.getElementById("filter_day").selectedIndex = x;
			document.getElementById("filter_day_label").innerText = days[x];
		} else if(x == 0 && epg.mode == "1"){
			document.getElementById("filter_day").selectedIndex = x;
			document.getElementById("filter_day_label").innerText = days[x];
		}
		document.getElementById("filter_day").options[x].text = days[x];
	}
	var now = new Date();
	lasttimestamp = now.getFullYear() + ":" + now.getMonth() + ":" + now.getDate();
}

function intialLoadComplete(feedtype){
	if(upd.criticalUpdateAvailable)
		return;

	if(feedtype == "popular")
		popLoaded = true;
	else if(feedtype == "mgu")
		mguLoaded = true;
	else if(feedtype == "favorites")
		favLoaded = true;
	else if(feedtype == "mgs")
		mgsLoaded = true;
	else if(feedtype == "epg")
		epgLoaded = true;
	else if(feedtype == "channels")
		chmLoaded = true;
	else if(feedtype == "devices")
		dvLoaded = true;
	else if(feedtype == "recommended")
		rcmLoaded = true;
	else if(feedtype == "keywords")
		keyLoaded = true;
	else if(feedtype == "networks")
		netLoaded = true;


	if(epgLoaded && chmLoaded && popLoaded && mguLoaded && mgsLoaded && favLoaded && dvLoaded && rcmLoaded && keyLoaded && netLoaded){
		ath.enableWidget();
		document.getElementById("frontSpinnerContainer").style.display = "none";
		initialSetupComplete = true;
		clearInterval(bigSpinnerTimer);
		bigSpinnerTimer = null;
	
		switch(parseInt(loadPref("selectedPage"), 10)){
			case 0:
				selectPage(0);
			break;
			case 1:
				selectPage(1);
			break;
			case 2:
				selectPage(2);
			break;
			case 3:
				selectPage(3);
			break;
			case 4:
				selectPage(4);
			break;
		}
		
		if(!mainTimer)
			mainTimer = setInterval("maintimerTick()", 60000);	
		setTimeout("checkFirstAuth()", 1000);
		setTimeout("upd.check(false, 0)", 1000);
	}
}

function checkFirstAuth(){
	if(epg.mode != "7" && parseInt(loadPref("shownSubscribeSheet"), 10) == 0 && parseInt(loadPref("firstAuth"), 10) == 1){
		savePref(1, "shownSubscribeSheet");
		document.getElementById("sheet-subscribe-moreinfo").onclick = function(){ widget.openURL("http://www.icetv.com.au/about.shtml");sht.closeSheet(3);}
		document.getElementById("sheet-subscribe-ok").onclick = function(){ sht.closeSheet(3);}
		
		document.getElementById("sheet-container").style.display = "block";
		document.getElementById("welcome-subscrive-sheet").style.display = "block";
		document.getElementById("welcome-subscrive-sheet").style.top = -parseInt(document.getElementById("welcome-subscrive-sheet").style.height, 10) + "px";
				
		var container;
	
		switch(selectedPage){
			case 0:
				container = document.getElementById("epg_container");
			break;
			case 1:
				container = document.getElementById("popular_container");
			break;
			case 2:
				if(mgu.isEmpty())
					container = document.getElementById("myguideupcoming_empty");
				else
					container = document.getElementById("myguideupcoming_container");
			break;
			case 3:
				if(mgs.isEmpty())
					container = document.getElementById("myguideseries_empty");
				else
					container = document.getElementById("myguideseries_container");
			break;
			case 4:
				if(rcm.isEmpty())
					container = document.getElementById("recommended_empty");
				else
					container = document.getElementById("recommended_container");
			break;
		}	
	
		container.style.opacity = 0.4;
		sht.showSheet(3, null);
	}
}

function maintimerTick(){
	if(ath.widgetDisabled)
		return;

	if(!initialSetupComplete)
		return;

	if(upd.criticalUpdateAvailable)
		return;

	var now = new Date();
	nowstamp = now.getFullYear() + ":" + now.getMonth() + ":" + now.getDate();
	
	if(lasttimestamp != nowstamp)
		updateDayMenu();

	switch(selectedPage){
		case 0:
			epg.updateDisplay();
			epg.updateData('false');
		break;
		case 1:
			popular.updateData('false');
		break;
		case 2:
			mgu.updateDisplay();
			mgu.updateData('false');
		break;
		case 3:
			mgs.updateData('false');
		break;
		case 4:
			rcm.updateData('false');
		break;
	}
	fav.updateData('false');
	dv.updateData('false');
	key.updateData('false');
}

function runWidget(){
	document.getElementById("frontSpinnerContainer").style.display = "block";
	document.getElementById("frontSpinnerContainer").style.opacity = 1;
	bigSpinnerTimer = setInterval("animatedBigSpinner()", 40);

	document.getElementById("widgetback").style.display = "none";
	document.getElementById("widgetback").style.opacity = 1;
	
	buildHourMenu();
	updateDayMenu();
	
	sht.setup();
	bck.setup();
	epg.setup();
	popular.setup();
	popular.buildDisplay();
	rcm.setup();
	upd.setup();

	mgs.setup();
	mgs.buildDisplay();

	mgu.setup();
	mgu.buildDisplay();

	window.resizeTo(window.outerWidth, 200);
    popularScroller = new AppleVerticalScrollbar(document.getElementById("popular_scroller"));
    popularScrollArea = new AppleScrollArea(document.getElementById("popular_content_container"));
   	popularScrollArea.addScrollbar(popularScroller);

    myguideScroller = new AppleVerticalScrollbar(document.getElementById("myguideupcoming_scroller"));
    myguideScrollArea = new AppleScrollArea(document.getElementById("myguideupcoming_content_container"));
   	myguideScrollArea.addScrollbar(myguideScroller);

    myguideSeriesScroller = new AppleVerticalScrollbar(document.getElementById("myguideseries_scroller"));
    myguideSeriesScrollArea = new AppleScrollArea(document.getElementById("myguideseries_content_container"));
   	myguideSeriesScrollArea.addScrollbar(myguideSeriesScroller);

    epgScroller = new AppleVerticalScrollbar(document.getElementById("epgsearch_scrollbar"));
    epgScrollArea = new AppleScrollArea(document.getElementById("epgsearch_content_container"));
   	epgScrollArea.addScrollbar(epgScroller);

   	recommendedScroller = new AppleVerticalScrollbar(document.getElementById("recommended_scroller"));
    recommendedScrollArea = new AppleScrollArea(document.getElementById("recommended_content_container"));
   	recommendedScrollArea.addScrollbar(recommendedScroller);

	document.addEventListener("keyup", keyUp, false);
	document.addEventListener("keydown", keyDown, false);
	window.onfocus = focus;
	window.onblur = blur;

	widget.onshow = dashVisible;
	widget.onhide = dashHidden;
	widget.onremove = widgetUninstalled;

	chmDownloader.buildURL(0, 'true');
	epgDownloader.buildURL(0, 'true');
	favDownloader.buildURL(0, 'true');
	mgsDownloader.buildURL(0, 'true');
	mguDownloader.buildURL(0, 'true');
	popularDownloader.buildURL(0, 'true');
	dvDownloader.buildURL(0, 'true');
	rcmDownloader.buildURL(0, 'true');
	keyDownloader.buildURL(0, 'true');
	netDownloader.buildURL(0, 'true');

	chmDownloader.fetchURL();
	epgDownloader.fetchURL();
	favDownloader.fetchURL();
	mgsDownloader.fetchURL();
	mguDownloader.fetchURL();
	popularDownloader.fetchURL();
	dvDownloader.fetchURL();
	rcmDownloader.fetchURL();
	keyDownloader.fetchURL();
	netDownloader.fetchURL();
}


function setup() {
	ath.setup();
	if(!loadPref("authToken")){
		ath.disableWidget();
		document.getElementById("sheet-container").style.display = "block";
		document.getElementById("welcome-sheet").style.display = "block";
		document.getElementById("sheet-container").style.height = document.getElementById("welcome-sheet").style.height;
		document.getElementById("widgetfront").style.height = (72 + parseInt(document.getElementById("welcome-sheet").style.height, 10)) + "px";
		window.resizeTo(window.outerWidth, (72 + parseInt(document.getElementById("welcome-sheet").style.height, 10)) + 30);
		document.getElementById("widgetback").style.display = "none";
		document.getElementById("widgetback").style.opacity = 1;
		return;
	}
	
	if(ath.isExpired()){
		ath.showExpiredSheet();
		return;	
	}

	setTimeout("runWidget();", 10);
}

function focus()
{
	document.getElementById("widgetfront").setAttribute("class", "");
	document.getElementById("widgetback").setAttribute("class", "");
}

function blur()
{
	document.getElementById("widgetfront").setAttribute("class", "nofocus");
	document.getElementById("widgetback").setAttribute("class", "nofocus");
}

function keyUp(event) 
{
	if(event.target == document.body)
	{
		if (event.metaKey || event.ctrlKey){
//			DEBUGLOG("ctrl pressed");
			return; 
		}
		
		window.focus()
		var realKey = String.fromCharCode(event.charCode).toLowerCase();

		/*** We want to use Esc or Enter to close a sheet but NOT if it is the critical update or auth expiry sheets.*/
		if( document.getElementById("critical-update-sheet").style.display == "none" && 
			document.getElementById("welcome-sheet").style.display == "none" && 
			document.getElementById("auth-expired-sheet").style.display == "none" &&
			document.getElementById("epg-sheet").style.display == "block" &&
			(event.charCode == 13 || event.charCode == 3 || event.charCode == 27)){
			if(sht.isActive && !sht.animator.active){
				sht.closeSheet(0)
			}
		}
		
		if(mainAnimationController.active)
			return;

		if((realKey == "1" || realKey == "t") && selectedPage != 0)
			selectPage(0);

		if((realKey == "2" || realKey == "p") && selectedPage != 1)
			selectPage(1);

		if((realKey == "3" || realKey == "u") && selectedPage != 2)
			selectPage(2);

		if((realKey == "4" || realKey == "m") && selectedPage != 3)
			selectPage(3);
			
		if((realKey == "5" || realKey == "r") && selectedPage != 4)
			selectPage(4);
		
		
		if(realKey == "n" && selectedPage == 0){
			resetDay();
			resetTime();
		}
		
		if(event.charCode == 63232){
			var date = new Date();
			date.setHours(3);
			sht.forceCloseSheets();
			var index = loadPref("currentFilterTime");
			if(index == -1){
				index = date.getHours() - 1;
			} else
				index--;
			if(date.getHours() == index)
				index = -1;
			
			savePref("" + (index) + "", "currentFilterTime");
			epg.updateFilterTime();
			document.getElementById("filter_time").selectedIndex = (index + 1);
			document.getElementById("filter_time_label").innerText = document.getElementById("filter_time").options[document.getElementById("filter_time").selectedIndex].text;
			document.getElementById("filter_time").blur();
		}
		if(event.charCode == 63233){
			var date = new Date();
			date.setHours(3);
			sht.forceCloseSheets();
			var index = loadPref("currentFilterTime");
			if(index == -1){
				index = date.getHours();
				if(index < 24)
					index++;
			} else
				index++;
			if(index == 24)
				index = -1;
			if(date.getHours() == index)
				index = -1;
				
			//index--;
			
			savePref("" + (index) + "", "currentFilterTime");
			epg.updateFilterTime();
			document.getElementById("filter_time").selectedIndex = (index + 1);
			document.getElementById("filter_time_label").innerText = document.getElementById("filter_time").options[document.getElementById("filter_time").selectedIndex].text;
			document.getElementById("filter_time").blur();
		}
		
		if(event.charCode == 63234){ // Back a page
			if(selectedPage == 1)
				selectPage(0);
			else if(selectedPage == 2)
				selectPage(1);
			else if(selectedPage == 3)
				selectPage(2)
			else if(selectedPage == 4)
				selectPage(3);
			else
				selectPage(4);
		}
		if(event.charCode == 63235){ // Foward a page
			if(selectedPage == 1)
				selectPage(2)
			else if(selectedPage == 2)
				selectPage(3);
			else if(selectedPage == 3)
				selectPage(4);
			else if(selectedPage == 4)
				selectPage(0);
			else 
				selectPage(1);
		}
		event.stopPropagation();
		event.preventDefault();
	}
}

function keyDown(event)
{
	if(event.target == document.body)
	{
		if (event.metaKey)
			return;
		event.stopPropagation();
		event.preventDefault();
	}
}

function showPrefs()
{
	prefsShowing = true;
	frontHeight = window.outerHeight;
	
	var front = document.getElementById("widgetfront");
	var back = document.getElementById("widgetback");

	// Set current list text to the value from nowAndNext
	document.getElementById('back-texttvguide').innerText = epg.visibleShows + ' programs';
	
	 // Set the currently selected item using nowAndNext 
	if(epg.visibleShows == "3")
		document.getElementById("now_and_next").options[0].selected = true;
	else if(epg.visibleShows == "5")
		document.getElementById("now_and_next").options[1].selected = true;


	 // Set the currently selected item using twentyFourHourClock 
	document.getElementById('back-text24hr').innerText = epg.timeFormat + ' hour clock';

	 // Set the currently selected item using twentyFourHourClock 
	if(epg.timeFormat == 12)
		document.getElementById("clock_mode").options[0].selected = true;
	else if(epg.timeFormat == 24)
		document.getElementById("clock_mode").options[1].selected = true;

	if(window.outerHeight < parseInt(back.style.height, 10))
		window.resizeTo(window.outerWidth, parseInt(back.style.height, 10));

	if (window.widget)
		widget.prepareForTransition("ToBack");
	
	document.getElementById("back-version").innerHTML = getVersionString(widgetVersion);
	
	front.style.display="none";
	back.style.display="block";

	if (window.widget)		
		setTimeout ('widget.performTransition();', 0);	
	backside = true;

	window.resizeTo(440, parseInt(back.style.height, 10) + 30);
	sht.forceCloseSheets();
}

function showFront()
{
	var front = document.getElementById("widgetfront");
	var back = document.getElementById("widgetback");

	frontHeight = activeSectionHeight() + 105;

	if(window.outerHeight < frontHeight)
		window.resizeTo(440, frontHeight);

	if (window.widget)
		widget.prepareForTransition("ToFront");
	
	front.style.display="block";
	back.style.display="none";

	if (window.widget)		
		setTimeout ('widget.performTransition();', 0);	
	backside = true;

	window.resizeTo(440, frontHeight);
	prefsShowing = false;
}

function checkForUpdate()
{
//	DEBUGLOG("core.js[743]: checkForUpdate()");
	showFront();
	isUpdateAvailable('true');
}

function showHelp()
{
//	DEBUGLOG("core.js[750]: showHelp()");
	widget.openURL(help_url);
}

function showMoreSettings()
{
//	DEBUGLOG("core.js[756]: showMoreSettings()");
	widget.openURL(settings_url);
}

function deauthoriseWidget()
{
//	DEBUGLOG("core.js[762]: deauthoriseWidget()");
	showFront();
	deleteAuthToken();
	savePref("", "authToken");
	savePref("", "authTokenExpiry");
	clearData();
	resetDisplay();
	setup();
}

function doRegister()
{
//	DEBUGLOG("core.js[774]: doRegister()");
	widget.openURL('http://www.icetv.com.au/cgi-bin/webwidget.cgi');
}

function dashVisible(){
	if(!mainTimer)
		mainTimer = setInterval("maintimerTick()", 60000);

	maintimerTick();
}

function dashHidden(){
	clearInterval(mainTimer);
	mainTimer = null
}

function widgetUninstalled()
{
//	DEBUGLOG("core.js[792]: widgetUninstalled()");
}

function getVersionString(versionString)
{
//	DEBUGLOG("core.js[797]: getVersionString('" + versionString +"')");
	/**
	 * The version string looks like XXYYZZZT
	 *
	 * Where
	 * XX  - Major Version Number
	 * YY  - Minor Version Number
	 * ZZZ - Build Number
	 * T   - Version Type ( A - Alpha, B - Beta, N - Normal )
	 */
	 
	var majorVersion = parseInt(versionString.substring(0,2), 10);
	var minorVersion = parseInt(versionString.substring(2,4), 10);
	var buildNumber = parseInt(versionString.substring(4,7), 10);
	var versionType = versionString.substring(7,8);
	var versionTypeString;
	
	if(versionType == "A")
		versionTypeString = "Alpha";
	else if(versionType == "B")
		versionTypeString = "Beta";
	else 
		versionTypeString = "Build";

	return majorVersion + "." + minorVersion + " (" + buildNumber + ")";
}

function selectPage(page){
	if(ath.widgetDisabled)
		return;

	if(!initialSetupComplete)
		return;
	
	if(selectedPage == page)
		return;
		
	sht.forceCloseSheets();
	
	if(mainAnimationController.active)
		mainAnimationController.cancel();
	
	savePref(page, "selectedPage");

	var outControl = null;
	var outElement = null;
	var outController = null;
	switch(selectedPage){
		case -1:
			outControl = document.getElementById("controls-filler");
		break;
		case 0:
			document.getElementById("mode-selector-epg").setAttribute("class", "mode-epg");
			outControl = document.getElementById("epg-controls");
			outElement = document.getElementById("epg_container");
			outController = epg;
		break;
		case 1:
			document.getElementById("mode-selector-popular").setAttribute("class", "mode-popular");
			outControl = document.getElementById("popular-controls");
			outElement = document.getElementById("popular_container");
			outController = popular;
		break;
		case 2:
			document.getElementById("mode-selector-myguideupcoming").setAttribute("class", "mode-myguideupcoming");
			outControl = document.getElementById("my-guide-upcoming-controls");
			if(mgu.isEmpty())
				outElement = document.getElementById("myguideupcoming_empty");
			else
				outElement = document.getElementById("myguideupcoming_container");
			outController = mgu;
		break;
		case 3:
			document.getElementById("mode-selector-myguideseries").setAttribute("class", "mode-myguideseries");
			outControl = document.getElementById("my-guide-series-controls");
			if(mgs.isEmpty())
				outElement = document.getElementById("myguideseries_empty");
			else
				outElement = document.getElementById("myguideseries_container");
			outController = mgs;
		break;
		case 4:
			document.getElementById("mode-selector-recommended").setAttribute("class", "mode-recommended");
			outControl = document.getElementById("recommended-controls");
			if(rcm.isDisabled())
				outElement = document.getElementById("recommendations_disabled");
			else if(rcm.isEmpty())
				outElement = document.getElementById("recommended_empty");
			else
				outElement = document.getElementById("recommended_container");
			outController = rcm;
		break;
	}	

	var inControl;
	var inElement;
	var inController;
	var toHeight = 0;
	switch(page){
		case 0:
			document.getElementById("mode-selector-epg").setAttribute("class", "mode-epg mode-sel");
			inControl = document.getElementById("epg-controls");
			inElement = document.getElementById("epg_container");
			toHeight = epg.animationHeight();
			inController = epg;
		break;
		case 1:
			document.getElementById("mode-selector-popular").setAttribute("class", "mode-popular mode-sel");
			inControl = document.getElementById("popular-controls");
			inElement = document.getElementById("popular_container");
			toHeight = popular.animationHeight();
			inController = popular;
		break;
		case 2:
			document.getElementById("mode-selector-myguideupcoming").setAttribute("class", "mode-myguideupcoming mode-sel");
			inControl = document.getElementById("my-guide-upcoming-controls");
			if(mgu.isEmpty())
				inElement = document.getElementById("myguideupcoming_empty");
			else
				inElement = document.getElementById("myguideupcoming_container");
			toHeight = mgu.animationHeight();
			inController = mgu;
		break;
		case 3:
			document.getElementById("mode-selector-myguideseries").setAttribute("class", "mode-myguideseries mode-sel");
			inControl = document.getElementById("my-guide-series-controls");
			if(mgs.isEmpty())
				inElement = document.getElementById("myguideseries_empty");
			else
				inElement = document.getElementById("myguideseries_container");
			toHeight = mgs.animationHeight();
			inController = mgs;
		break;
		case 4:
			document.getElementById("mode-selector-recommended").setAttribute("class", "mode-recommended mode-sel");
			inControl = document.getElementById("recommended-controls");
			if(rcm.isDisabled())
				inElement = document.getElementById("recommendations_disabled");
			else if(rcm.isEmpty())
				inElement = document.getElementById("recommended_empty");
			else
				inElement = document.getElementById("recommended_container");
			toHeight = rcm.animationHeight();
			inController = rcm;
		break;
	}
	
	selectedPage = page;

	inControl.style.display = "block"
	inControl.style.top = "0px";
	inControl.style.top = "25px";
	document.getElementById("controler-inner-container").style.top = "-25px"

	inElement.style.opacity = 0;
	if(outElement)
		outElement.style.opacity = 1;
	
	inElement.style.display = "block"
	inController.resize();

	inController.isSelected = true;
	if(outController)
		outController.isSelected = false;
	
	inControl.style.top = "0px";
	outControl.style.top = "25px";
	
	mainAnimationController.purge();
	
	var bar = new AnimationObject(document.getElementById("controler-inner-container"));
	bar.setOptions(0, 18);
	bar.pushPhase(kAnimatorMoveVerticalTop, -25, 0, 18, 0, null)

	if(outElement){
		var fadeOut = new AnimationObject(outElement);
		fadeOut.setOptions(kAnimateOptionsHide, 3);
		fadeOut.pushPhase(kAnimateOpacity, 1, 0, 3, 0, null)
		mainAnimationController.pushGroup(Array(fadeOut));
	}
	
	var fadeIn = new AnimationObject(inElement);
	fadeIn.setOptions(0, 18);
	fadeIn.pushPhase(kAnimateOpacity, 0, 1, 3, 15, null)

	var startHeight = parseInt(document.defaultView.getComputedStyle(document.getElementById("main_container"),"").getPropertyValue("height"), 10);
	var resize = new AnimationObject(document.getElementById("main_container"));
	resize.setOptions(0, 18);
	resize.pushPhase(kAnimatorResizeHeight, startHeight, toHeight, 12, 3, null)

	var startPosition = parseInt(document.defaultView.getComputedStyle(document.getElementById("main_base"),"").getPropertyValue("top"), 10)
	var move = new AnimationObject(document.getElementById("main_base"));
	move.setOptions(0, 18);
	move.pushPhase(kAnimatorMoveVerticalTop, startPosition, toHeight + 72, 12, 3, null)
			
	
	var hide = new AnimationObject(outControl);
	hide.setOptions(kAnimateOptionsHide, 18);
	hide.pushPhase(0, 0, 0, 17, 1, null)

	mainAnimationController.purgeCallbacks();
	mainAnimationController.pushGroup(Array(bar, fadeIn, resize, move, hide));
	mainAnimationController.prepare(300, 18);
	mainAnimationController.pushCallback(function(){ resizeWindowAfterPageSwitch(); inController.updateData('false');});

	if(window.outerHeight < (toHeight + 135))
		window.resizeTo(window.outerWidth, toHeight + 135);

	mainAnimationController.run();

    /**
     * Force update on page change, not including EPG (too large)
     */
	switch(selectedPage){
		case 0:
			mgu.updateData('true');
			epg.updateDisplay();
		break;
		case 1:
			popular.updateData('true');
		break;
		case 2:
			mgu.updateData('true');
		break;
		case 3:
			mgs.updateData('true');
		break;
		case 4:
			rcm.updateData('true');
		break;
	}
	fav.updateData('true');
	dv.updateData('true');
	key.updateData('true');
}

function resizeFromHeightReset(oldContainerHeight, container){
	var newContainerHeight = parseInt(container.style.height, 10);
	container.style.height = oldContainerHeight + "px"
	
	var outElement;
	var outController;
	switch(selectedPage){
		case 0:
			outElement = document.getElementById("epg_container");
			outController = epg;
		break;
		case 1:
			outElement = document.getElementById("popular_container");
			outController = popular;
		break;
		case 2:
			outElement = document.getElementById("myguideupcoming_container");
			outController = mgu;
		break;
		case 3:
			outElement = document.getElementById("myguideseries_container");
			outController = mgs;
		break;
		case 4:
			outElement = document.getElementById("recommended_container");
			outController = rcm;
		break;
	}	
	
		
	mainAnimationController.cancel();
	mainAnimationController.purge();
	
	var toheight = outController.animationHeight();

	var startHeight = parseInt(document.getElementById("main_container").style.height, 10);
	var resize = new AnimationObject(document.getElementById("main_container"));
	resize.setOptions(0, 10);
	resize.pushPhase(kAnimatorResizeHeight, startHeight, toheight, 10, 0, null)

	var resize2 = new AnimationObject(container);
	resize2.setOptions(0, 10);
	resize2.pushPhase(kAnimatorResizeHeight, oldContainerHeight, newContainerHeight, 10, 0, null)

	var startPosition = parseInt(document.getElementById("main_base").style.top, 10)
	var move = new AnimationObject(document.getElementById("main_base"));
	move.setOptions(0, 18);
	move.pushPhase(kAnimatorMoveVerticalTop, startPosition, toheight + 72, 10, 0, null)
			

	mainAnimationController.purgeCallbacks();
	mainAnimationController.pushGroup(Array(resize, move, resize2));
	mainAnimationController.prepare(150, 10);
	var _self = this;
	mainAnimationController.pushCallback(function(){ outController.resize(); });

	if(window.outerHeight < (toheight + 135))
		window.resizeTo(window.outerWidth, toheight + 135);

	mainAnimationController.run();
}

function resizeEpgSearch(from){
	var inElement;
	var outElement;
	if(from){
		inElement = document.getElementById("epg_guide_content")
		outElement = document.getElementById("epgsearch_container")
	} else {
		inElement = document.getElementById("epgsearch_container");
		outElement = document.getElementById("epg_guide_content")
	}
	
	inElement.style.opacity = 0;
	inElement.style.display = "block"
	
	//mainAnimationController.cancel();
	mainAnimationController.purge();
	
	var fadeOut = new AnimationObject(outElement);
	fadeOut.setOptions(kAnimateOptionsHide, 3);
	fadeOut.pushPhase(kAnimateOpacity, 1, 0, 3, 0, null)

	var fadeIn = new AnimationObject(inElement);
	fadeIn.setOptions(0, 18);
	fadeIn.pushPhase(kAnimateOpacity, 0, 1, 3, 15, null)

	var startHeight = parseInt(document.getElementById("main_container").style.height, 10);
	var resize = new AnimationObject(document.getElementById("main_container"));
	resize.setOptions(0, 18);
	resize.pushPhase(kAnimatorResizeHeight, startHeight, epg.animationHeight(), 12, 3, null)

	var startPosition = parseInt(document.getElementById("main_base").style.top, 10)
	var move = new AnimationObject(document.getElementById("main_base"));
	move.setOptions(0, 18);
	move.pushPhase(kAnimatorMoveVerticalTop, startPosition, epg.animationHeight() + 72, 12, 3, null)
			
	
	var hide = new AnimationObject(outElement);
	hide.setOptions(kAnimateOptionsHide, 18);
	hide.pushPhase(0, 0, 0, 17, 1, null)

	mainAnimationController.pushGroup(Array(fadeOut, fadeIn, resize, move, hide));
	mainAnimationController.prepare(300, 18);
	var _self = this;
	mainAnimationController.pushCallback(function(){ _self.finishedPageSwitch(); });

	if(window.outerHeight < (epg.animationHeight() + 105))
		window.resizeTo(window.outerWidth, epg.animationHeight() + 105);
	mainAnimationController.run();
}

function finishedPageSwitch(){
	var outController;
	switch(selectedPage){
		case 0:
			outController = epg;
		break;
		case 1:
			outController = popular;
		break;
		case 2:
			outController = mgu;
		break;
		case 3:
			outController = mgs;
		break;
		case 4:
			outController = rcm;
		break;
	}
	if(outController)
		outController.resize();
}

var tooltipsActive = false;

function showtooltip(tooltip){
	if(!initialSetupComplete)
		return;
	if(ath.widgetDisabled)
		return;
	
	if(tooltipAnimationController.active){
		tooltipAnimationController.cancel();
	}

	var element = null;
	switch(tooltip){
		case 0:
			element = document.getElementById("mode-tooltip-epg");
		break;
		case 1:
			element = document.getElementById("mode-tooltip-popular");
		break;
		case 2:
			element = document.getElementById("mode-tooltip-myguide-upcoming");
		break;
		case 3:
			element = document.getElementById("mode-tooltip-myguide-series");
		break;
		case 4:
			element = document.getElementById("mode-tooltip-recommended");
		break;
		case 5:
			element = document.getElementById("mode-tooltip-prefs");
		break;
	}
	element.style.zIndex = 101;
	element.style.opacity = 1;
	element.style.display = "block";
}

function hidetooltip(tooltip){
	if(tooltipAnimationController.active){
		tooltipAnimationController.cancel();
	}
	

	var element = null;
	switch(tooltip){
		case 0:
			element = document.getElementById("mode-tooltip-epg");
		break;
		case 1:
			element = document.getElementById("mode-tooltip-popular");
		break;
		case 2:
			element = document.getElementById("mode-tooltip-myguide-upcoming");
		break;
		case 3:
			element = document.getElementById("mode-tooltip-myguide-series");
		break;
		case 4:
			element = document.getElementById("mode-tooltip-recommended");
		break;
		case 5:
			element = document.getElementById("mode-tooltip-prefs");
		break;
	}	

	if(!tooltipsActive){
		tooltipAnimationController.purge();
	
		var fadeOut = new AnimationObject(element);
		fadeOut.setOptions(kAnimateOptionsResetZIndex, 10);
		fadeOut.pushPhase(kAnimateOpacity, 1, 0, 10, 0, null)
		
		tooltipAnimationController.pushGroup(Array(fadeOut));
		tooltipAnimationController.prepare(350, 10);
		tooltipAnimationController.run();
	}
}

function buildHourMenu(){
	document.getElementById("filter_time").options[0].text = "Now";
	for(x=0;x<24;x++){
		if(epg.timeFormat == 12){
			if(x == 0)
				document.getElementById("filter_time").options[x + 1].text = "12 AM";
			else if(x < 12)
				document.getElementById("filter_time").options[x + 1].text = x + " AM";
			else if(x == 12)
				document.getElementById("filter_time").options[x + 1].text = "12 PM";
			else
				document.getElementById("filter_time").options[x + 1].text = (x - 12) + " PM";
		} else {
			document.getElementById("filter_time").options[x + 1].text = x + ":00";
		}	
		
	}
	for(x=0;x<25;x++){
		if((x - 1) == loadPref("currentFilterTime")){
			document.getElementById("filter_time_label").innerText = document.getElementById("filter_time").options[x].text;
			document.getElementById("filter_time").selectedIndex = x;
		}
	}
}

function resetTime(){
	sht.forceCloseSheets();
	document.getElementById("filter_time").selectedIndex = 0;
	document.getElementById("filter_time_label").innerText = document.getElementById("filter_time").options[document.getElementById("filter_time").selectedIndex].text;
	savePref("-1", "currentFilterTime");
	epg.updateFilterTime();
}

function selectTime(){
	sht.forceCloseSheets();
	var index = document.getElementById("filter_time").selectedIndex - 1
	savePref("" + index + "", "currentFilterTime");
	epg.updateFilterTime();
	document.getElementById("filter_time_label").innerText = document.getElementById("filter_time").options[document.getElementById("filter_time").selectedIndex].text;
	document.getElementById("filter_time").blur();
}

function resetDay(){
	sht.forceCloseSheets();
	document.getElementById("filter_day").selectedIndex = 0;
	document.getElementById("filter_day_label").innerText = document.getElementById("filter_day").options[document.getElementById("filter_day").selectedIndex].text;
	savePref("0", "currentFilterDay");
	epg.updateFilterDay();
}

function selectDay(){
	var index = document.getElementById("filter_day").selectedIndex
	if(index != 0 && epg.mode != "7"){
		if(sht.extraDaysSheetActive)
			return;
		
		sht.forceCloseSheets();
		document.getElementById("extradays-sheet-subscribe-moreinfo").onclick = function(){ widget.openURL("http://www.icetv.com.au/about.shtml");sht.closeSheet(2);}
		document.getElementById("extradays-sheet-subscribe-ok").onclick = function(){ sht.closeSheet(2);}
	
		document.getElementById("sheet-container").style.display = "block";
		document.getElementById("extraDays_sheet").style.top = -parseInt(document.getElementById("extraDays_sheet").style.height, 10) +"px";
		document.getElementById("extraDays_sheet").style.display = "block";
		document.getElementById("sheet-container").style.height = document.getElementById("extraDays_sheet").style.height;

		var container = document.getElementById("epg_container");
		container.style.opacity = 0.4;

		sht.showSheet(2, null);
		document.getElementById("filter_day").selectedIndex = 0;
		return;
	}
	if(!sht.extraDaysSheetActive)
		sht.forceCloseSheets();
	
	savePref("" + index + "", "currentFilterDay");
	epg.updateFilterDay();
	document.getElementById("filter_day_label").innerText = document.getElementById("filter_day").options[document.getElementById("filter_day").selectedIndex].text;
}

function resizeFront(toHeight){
	if(prefsShowing)
		return;
	var sheetHeight = sht.activeHeight();
	if(sheetHeight != -1){
		if(toHeight < sheetHeight)
			return
		if(parseInt(document.getElementById("widgetfront").style.height, 10) < toHeight)
			document.getElementById("widgetfront").style.height = toHeight + "px";	
	}
	
	if(window.outerHeight < (toHeight + 30)){
		document.getElementById("widgetfront").style.height = toHeight + "px";	
		window.resizeTo(window.outerWidth, toHeight + 30);
	} else {
	}
}

function activeSectionHeight(){
	var toHeight = -1;
	switch(selectedPage){
		case 0:
			toHeight = epg.animationHeight();
		break;
		case 1:
			toHeight = popular.animationHeight();
		break;
		case 2:
			toHeight = mgu.animationHeight();
		break;
		case 3:
			toHeight = mgs.animationHeight();
		break;
		case 4:
			toHeight = rcm.animationHeight();
		break;
	}
	return toHeight;
}

function resizeWindowAfterPageSwitch(){
	var toHeight = activeSectionHeight();
//    DEBUGLOG("core.js[1401]: toHeight: " + toHeight);
	if(toHeight != -1){
		document.getElementById("widgetfront").style.height = toHeight + 135 + "px";	
		window.resizeTo(window.outerWidth, toHeight + 135);
//        DEBUGLOG("core.js[1405]: window.resizeTo(" + window.outerWidth + ", " + (toHeight+135) + ");");
	}
}

function getFilterDays()
{
	var currentDate = new Date()
	var currentDay = currentDate.getDay()
	var days = Array();
	
	/**
	 * Get the number of days of EPG for the subscriber, widget accounts get "1", normal accounts get "7"
	 * default to "7"
	 */
	//MUST FIX
	/////////////
	var numDaysEpg = "7";//(getEpgHeader['num_days_epg']) ? getEpgHeader['num_days_epg'] : "7";
	
	var dayNameArray = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
	
	/**
	 * Widget Accounts only get data for today
	 */
	if(numDaysEpg == "1")
	{
		days[0] = "Today";
		return days;
	}

	/**
	 * Normal Accounts only get data for 7 days
	 */
	else
	{
		/**
		 * Start with "Today" & "Tomorrow"
		 */
		days[0] = "Today";
		days[1] = "Tomorrow";
		for(var i=2;i<7;i++)
		{
			days[i] = dayNameArray[(currentDay+i)%7];
			//filterDaysArray[i] = dayRec;
		}
	}
	return days;
}

function decodeFlags(flags)
{
	var flagArray = Array();
	
	if(flags & 1)
		flagArray.push("BW");
		
	if(flags & 2)
		flagArray.push("CC");
		
	if(flags & 4)
		flagArray.push("HD");
		
	if(flags & 8)
		flagArray.push("Rpt");
		
	if(flags & 16)
		flagArray.push("WS");
		
	if(flags & 32)
		flagArray.push("L");
		
	if(flags & 64)
		flagArray.push("V");

	if(flags & 128)
		flagArray.push("D");

	if(flags & 256)
		flagArray.push("S");

	if(flags & 512)
		flagArray.push("G");

	return flagArray;
}

function decodeRating(rating)
{
	if(rating == 0)
		return "G";
	else if(rating == 9)
		return "PG";
	else if(rating == 12)
		return "M";
	else if(rating == 15)
		return "MA";
	else if(rating == 18)
		return "AV";
	else
		return "??";
}

function generateAuthenticateUrl(tempToken)
{
	api_check = hex_sha1(api_secret + 'temp_token' + tempToken );
	tempTokenUrl = host_url + '?op=' + 'login' + '&api_key=' + api_key + '&temp_token=' + tempToken + '&api_check=' + api_check; 
	return tempTokenUrl;
}

function setTimeFormat(){
	var format;
	if(document.getElementById("clock_mode").selectedIndex == 0)
		format = 12;
	else
		format = 24;
	savePref(format, "twentyFourHourClock");
	document.getElementById("back-text24hr").innerText = format + " hour clock"
	
	epg.setTimeFormat(format);
	mgu.updateDisplay();
	buildHourMenu();
}

function smartSplit( strDelimiters, strData, bWantBlankFields, strQuoteChar, bStripQuoteChar )
{
  //  Splits a string with a given set of delimiter characters into an array of strings.
  //  There can be multiple delimiters, but splitting is done any time an individual
  //  delimiter character is found (delimiter phrases aren't supported).
  //  The strQuoteChar would typically be a quote or double-quote character. When this
  //  is included, multiple keywords wrapped in the quote char are returned as one.
  //  So for example, [one two "three four" five] returns [one, two, three four, five],
  //  with the quotes removed.
      
    var vFields = new Array;
    var nLen = strData.length;
    var prev = 0;
    var bFirst = 1;
    var bInToken = 1;
    var nIndex;
    var nIndex2;
    var strField;
    var i;
    
  for(i = 0; i < nLen; i++)
  {     
    if(strDelimiters.indexOf(strData[i]) != -1)
    {       
      // Found a delimiter.
      if(i == 0)
      { 
        if(bWantBlankFields)
        {
          // At beginning of string, add an empty string.
          vFields.push("");
        } 
        bFirst = false;
      } 
      else
      {
        // Not at beginning of string.
        if(bFirst == true)
        {
          // First time adding a string, token starts at beginning of string.
          vFields.push(strData.substr(0, i));
          bFirst = false;
        }
        else if(i == prev)
        {
          if(bWantBlankFields)
          {
            // Two delimiters in a row, add an empty string.
            vFields.push("");
          }
        }
        else
        {
          // Normal, token is from prev to current.
          vFields.push(strData.substr(prev, i - prev));
        }
      }
      // Move prev to next char after delimiter.
      prev = i + 1;
    }
    else if(strData.substr(i, 1) == strQuoteChar)
    {
      // Found the beginning of a quoted phrase. Find the end. [one two "three four" five]
      if(i < nLen + 1 && (nIndex = strData.indexOf(strQuoteChar, i + 1)) != -1)
      {
        // Found the matching quote at the end of the phrase.
        // Get everything including the quotes.
        //   Use prev as the start, in case something's before the first quote [-"two three"]
        strField = strData.substr(prev, nIndex - prev + 1);
        strField = strField.replace(new RegExp(strQuoteChar, "gi"), "");

        vFields.push(strField);
        bFirst = false;
        // Move prev to beginning of next token.
        prev = nIndex + 1;
        while(prev < nLen && strDelimiters.indexOf(strData[prev]) != -1)
          ++prev;
        // Move iterator to last delimiter before the new token.
        i = prev - 1;
        bInToken = false;
      }
      // else, No matching quote, just ignore it and continue.
    }
    else
    {
      // Normal, non-delimiter character.
      bInToken = true;
    }
  }
  // Handle last token if we're in the middle of one.
  if(i != 0 && bInToken)
  {
    if(prev < nLen)
    {
      // Normal, token is from prev to end of string.
      vFields.push(strData.substr(prev));
    }
    else if(bWantBlankFields)
    {
      // String ends with a delimiter, add an empty token.
      vFields.push("");
    }
  }

  return vFields;
}
